<?php

namespace app\models;

use Yii;
use DateTime;
use DateInterval;
use yii\base\Model;
use app\models\Paiement;
use yii\httpclient\Client;
use yii\helpers\{VarDumper, Json};
//use yii\base\Model;

/**
 * AbonnementForm is the model behind the Subscription (user/abonnement) form.
 */
class PaiementForm extends Model
{
	const TID_REGEX = '/[A-Za-z]{2,}[0-9]{6}\.[0-9]{4}\.[A-Za-z][0-9]{5,}/';

	public $transaction_id;

    /**
     * @return array the validation rules.
     */
    public function rules()
    {
        return [
	        [['transaction_id'], 'trim'],
	        [['transaction_id'], 'required'],
					// ne plus limiter à 20 puisqu'on accepte un SMS entier à interpréter
	        // ['transaction_id', 'string', 'length' => 20],
					// mettre en majuscule un SMS entier peut être long. mieux vaut le faire dans fetch()
					// après avoir extrait le transaction id
					// ['transaction_id', 'filter', 'filter' => function ($value) {
					// 	return strtoupper($value);
					// }],
					['transaction_id', 'match', 'pattern' => PaiementForm::TID_REGEX],
      //    ['transaction_id', 'fetch'],
        	// ['transaction_id', 'exist', 'targetAttribute' => 'transaction_id',
          //                             'targetClass' => 'common\models\Paiement',
          //                             'filter' => ['statut' => Paiement::STATUS_PENDING,
					// 																				 'user' => $myUser->id],
					// 													  'message' => 'Cet ID de Transaction n\'existe pas'],
        ];
    }

    /**
     * @return array customized attribute labels
     */
    public function attributeLabels()
    {
        return [
        	'transaction_id' => Yii::t('app','ID Transaction'),
        ];
    }

		/**
		* Vérifie l'existence de l'ID transaction sur le serveur et récupère les données
    * de l'opération le cas échéant
		* @param string $accessKey clé d'accès à l'API
		* @param array $montants les montants attendus
		* @param array $immutableRowIds tableau des id d'enregistrement à ne pas modifier
		* @return Paiement le paiement, s'il est trouvé
		*/
    public function fetch($accessKey, $montants = [], $immutableRowIds = [])
    {
		$matches=array();
		preg_match(PaiementForm::TID_REGEX, $this->transaction_id, $matches);
		$this->transaction_id=strtoupper($matches[0]);

        $client = new Client();
        $response = $client->createRequest()
            ->setMethod('GET')
            ->setUrl(MPAY_API_BASE_URL.'/paiement/index')
            ->setData(['PaiementSearch[transaction_id]' => $this->transaction_id])
            ->addHeaders(['Authorization' => 'Basic ' . base64_encode($accessKey . ':')])
            ->send();
		Yii::debug('response->content = ' . VarDumper::dumpAsString($response->content),__METHOD__);
        if ($response->isOk && $response->content!=='[]') {
			$data = Json::decode($response->content, $asArray = true)[0];
			if($data['statut']=='Pending') {
            //on sauvegarde l'id en provenance de l'API, et on l'efface afin que l'objet
            //Paiement génère le sien au moment de la sauvegarde
			$remoteId= $data['id'];
				$fields=array();
							$fieldList=null;
							$paiement=null;
							// if(defined('SAVE_LOCAL_RECORD') && SAVE_LOCAL_RECORD)
									$fieldList=array_keys((new Paiement())->attributes);
							// else
							// 	$fieldList=Paiement::CHAMPS;
				foreach ( $fieldList as $fieldname ) {
				$fields[$fieldname] = $data[$fieldname];
				}
							// if(defined('SAVE_LOCAL_RECORD') && SAVE_LOCAL_RECORD)
								$paiement=new Paiement($fields);
							// else
				// 	$paiement= new Model($fields);
							//a faire éventuellement dans le controller avant enregistrement
				//$paiement->id=null;
							//vérification du montant
							//faire cette vérification avant la création de l'objet Paiement serait
							//plus rapide mais moins lisible
							if(!in_array($paiement->amount, $montants)) {
									$this->addError('transaction_id', Yii::t('app', 'Le montant ne correspond pas.'));
									return false;
							}
				//On met le statut à OK (si la config le demande)
							if(defined('UPDATE_REMOTE_RECORD') && UPDATE_REMOTE_RECORD && !in_array($remoteId,$immutableRowIds)) {
					//TODO Vérifier que l'opération a abouti
					$client->createRequest()
						->setMethod('POST')
						->setUrl(MPAY_API_BASE_URL.'/paiement/statut')
						->setData(['id' => $remoteId, 'statut' => 'OK'])
						->addHeaders(['Authorization' => 'Basic ' . base64_encode($accessKey . ':')])
						->send();
							}
				return $paiement;
			} else { //statut !== pending
				$this->addError('transaction_id', Yii::t('app', 'La transaction "' . $this->transaction_id .  '" est déjà finalisée.'));
				return false;
			}
        } else { //paiement not found
						$this->addError('transaction_id', Yii::t('app', 'La transaction "' . $this->transaction_id .  '" n\'existe pas.'));
						return false;
        }
    }

}
